@@ -184,10 +184,6 @@ JS_STATIC_DLL_CALLBACK(JSBool)
 XPC_NW_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
                 jsval *rval);
 
-JS_STATIC_DLL_CALLBACK(JSBool)
-XPCNativeWrapperCtor(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
-                     jsval *rval);
-
 static inline
 JSBool
 ThrowException(nsresult ex, JSContext *cx)
@@ -784,7 +780,7 @@ MirrorWrappedNativeParent(JSContext *cx, XPCWrappedNative *wrapper,
   return JS_TRUE;
 }
 
-JS_STATIC_DLL_CALLBACK(JSBool)
+JSBool JS_DLL_CALLBACK
 XPCNativeWrapperCtor(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
                      jsval *rval)
 {
@@ -1084,22 +1080,18 @@ XPCNativeWrapper::AttachNewConstructorObject(XPCCallContext &ccx,
 // static
 JSObject *
 XPCNativeWrapper::GetNewOrUsed(JSContext *cx, XPCWrappedNative *wrapper,
-                               JSObject *callee)
+                               nsIPrincipal *aObjectPrincipal)
 {
-  if (callee) {
-    nsCOMPtr<nsIPrincipal> prin;
-
+  if (aObjectPrincipal) {
     nsIScriptSecurityManager *ssm = XPCWrapper::GetSecurityManager();
-    nsresult rv = ssm->GetObjectPrincipal(cx, callee, getter_AddRefs(prin));
-    if (NS_SUCCEEDED(rv) && prin) {
-      PRBool isSystem;
-      rv = ssm->IsSystemPrincipal(prin, &isSystem);
-      if (NS_SUCCEEDED(rv) && !isSystem) {
-        jsval v = OBJECT_TO_JSVAL(wrapper->GetFlatJSObject());
-        if (!XPCNativeWrapperCtor(cx, JSVAL_TO_OBJECT(v), 1, &v, &v))
-          return nsnull;
-        return JSVAL_TO_OBJECT(v);
-      }
+
+    PRBool isSystem;
+    nsresult rv = ssm->IsSystemPrincipal(aObjectPrincipal, &isSystem);
+    if (NS_SUCCEEDED(rv) && !isSystem) {
+      jsval v = OBJECT_TO_JSVAL(wrapper->GetFlatJSObject());
+      if (!XPCNativeWrapperCtor(cx, JSVAL_TO_OBJECT(v), 1, &v, &v))
+        return nsnull;
+      return JSVAL_TO_OBJECT(v);
     }
   }
 
