@@ -1,40 +1,42 @@
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Google Safe Browsing.
- *
- * The Initial Developer of the Original Code is Google Inc.
- * Portions created by the Initial Developer are Copyright (C) 2006
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Fritz Schneider <fritz@google.com> (original author)
- *   Annie Sullivan <sullivan@google.com>
- *   Aaron Boodman <aa@google.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+# ***** BEGIN LICENSE BLOCK *****
+# Version: MPL 1.1/GPL 2.0/LGPL 2.1
+#
+# The contents of this file are subject to the Mozilla Public License Version
+# 1.1 (the "License"); you may not use this file except in compliance with
+# the License. You may obtain a copy of the License at
+# http://www.mozilla.org/MPL/
+#
+# Software distributed under the License is distributed on an "AS IS" basis,
+# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+# for the specific language governing rights and limitations under the
+# License.
+#
+# The Original Code is Google Safe Browsing.
+#
+# The Initial Developer of the Original Code is Google Inc.
+# Portions created by the Initial Developer are Copyright (C) 2006
+# the Initial Developer. All Rights Reserved.
+#
+# Contributor(s):
+#   Fritz Schneider <fritz@google.com> (original author)
+#   Annie Sullivan <sullivan@google.com>
+#   Aaron Boodman <aa@google.com>
+#
+# Alternatively, the contents of this file may be used under the terms of
+# either the GNU General Public License Version 2 or later (the "GPL"), or
+# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+# in which case the provisions of the GPL or the LGPL are applicable instead
+# of those above. If you wish to allow use of your version of this file only
+# under the terms of either the GPL or the LGPL, and not to allow others to
+# use your version of this file under the terms of the MPL, indicate your
+# decision by deleting the provisions above and replace them with the notice
+# and other provisions required by the GPL or the LGPL. If you do not delete
+# the provisions above, a recipient may use your version of this file under
+# the terms of any one of the MPL, the GPL or the LGPL.
+#
+# ***** END LICENSE BLOCK *****
+
+#ifdef DEBUG
 
 // Generic logging/debugging functionality that:
 //
@@ -88,7 +90,6 @@
 //
 // See classes below for more methods. 
 //
-// TODO add abililty to alert() instead of dump()? Should be easy.
 // TODO add code to set prefs when not found to the default value of a tristate
 // TODO add error level support
 // TODO add ability to turn off console output
@@ -126,16 +127,12 @@ function G_DebugL(who, msg) {
 
     if (zone.zoneIsEnabled()) {
       G_debugService.dump(
-        G_File.LINE_END_CHAR +
-        "************************************************************" +
-        G_File.LINE_END_CHAR);
+        "\n************************************************************\n");
 
       G_Debug(who, msg);
 
       G_debugService.dump(
-        "************************************************************" +
-        G_File.LINE_END_CHAR +
-        G_File.LINE_END_CHAR);
+        "************************************************************\n\n");
     }
   }
 }
@@ -150,7 +147,7 @@ function G_DebugL(who, msg) {
 function G_TraceCall(who, msg) {
   if (G_GDEBUG) {
     if (G_debugService.callTracingEnabled()) {
-      G_debugService.dump(msg + G_File.LINE_END_CHAR);
+      G_debugService.dump(msg + "\n");
     }
   }
 }
@@ -182,17 +179,6 @@ function G_Assert(who, condition, msg) {
   }
 }
 
-/**
- * Assert two things are equal (as in ==).
- */
-function G_AssertEqual(who, expected, actual, msg) {
-  if (G_GDEBUG) {
-    G_GetDebugZone(who).assert(
-        expected == actual,
-        msg + " Expected: {%s}, got: {%s}".subs(expected, actual));
-  }
-}
-
 /**
  * Helper function that takes input and returns the DebugZone
  * corresponding to it.
@@ -207,7 +193,7 @@ function G_GetDebugZone(who) {
 
     if (who && who.debugZone) {
       zone = who.debugZone;
-    } else if (isString(who)) {
+    } else if (typeof who == "string") {
       zone = who;
     }
 
@@ -283,9 +269,7 @@ G_DebugZone.prototype.disableZone = function() {
 G_DebugZone.prototype.debug = function(msg) {
   if (G_GDEBUG) {
     if (this.zoneIsEnabled()) {
-      this.debugService_.dump("[%s] %s%s".subs(this.zone_,
-                                               msg,
-                                               G_File.LINE_END_CHAR));
+      this.debugService_.dump("[" + this.zone_ + "] " + msg + "\n");
     }
   }
 }
@@ -297,9 +281,7 @@ G_DebugZone.prototype.debug = function(msg) {
  */
 G_DebugZone.prototype.error = function(msg) {
   if (G_GDEBUG) {
-    this.debugService_.dump("[%s] %s%s".subs(this.zone_,
-                                             msg,
-                                             G_File.LINE_END_CHAR));
+    this.debugService_.dump("[" + this.zone_ + "] " + msg + "\n");
     throw new Error(msg);
     debugger;
   }
@@ -341,12 +323,6 @@ function G_DebugService(opt_prefix) {
 
     this.loggifier = new G_Loggifier();
     this.settings_ = new G_DebugSettings();
-
-    // We observe the console service so that we can echo errors that get 
-    // reported there to the file log.
-    Cc["@mozilla.org/consoleservice;1"]
-      .getService(Ci.nsIConsoleService)
-      .registerListener(this);
   }
 }
 
@@ -562,8 +538,7 @@ G_DebugService.prototype.dump = function(msg) {
                       .getService(Components.interfaces.nsIConsoleService);
         console.logStringMessage(msg);
       } catch(e) {
-        dump("G_DebugZone ERROR: COULD NOT DUMP TO CONSOLE" +
-             G_File.LINE_END_CHAR);
+        dump("G_DebugZone ERROR: COULD NOT DUMP TO CONSOLE\n");
       }
     }
 
@@ -576,11 +551,25 @@ G_DebugService.prototype.dump = function(msg) {
  */
 G_DebugService.prototype.maybeDumpToFile = function(msg) {
   if (this.logFileIsEnabled() && this.logFile_) {
-    if (!this.logWriter_) {
-      this.logWriter_ = new G_FileWriter(this.logFile_, true);
-    }
 
-    this.logWriter_.write(msg);
+    /* try to get the correct line end character for this platform */
+    if (!this._LINE_END_CHAR)
+      this._LINE_END_CHAR =
+        Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULRuntime)
+                                         .OS == "WINNT" ? "\r\n" : "\n";
+    if (this._LINE_END_CHAR != "\n")
+      msg = msg.replace(/\n/g, this._LINE_END_CHAR);
+
+    try {
+      var stream = Cc["@mozilla.org/network/file-output-stream;1"]
+                   .createInstance(Ci.nsIFileOutputStream);
+      stream.init(this.logFile_,
+                  0x02 | 0x08 | 0x10 /* PR_WRONLY | PR_CREATE_FILE | PR_APPEND */
+                  -1 /* default perms */, 0 /* no special behavior */);
+      stream.write(msg, msg.length);
+    } finally {
+      stream.close();
+    }
   }
 }
 
@@ -599,7 +588,7 @@ G_DebugService.prototype.observe = function(consoleMessage) {
       // Only report these messages if the error level is INFO.
       if (errorLevel == G_DebugService.ERROR_LEVEL_INFO) {
         this.maybeDumpToFile(G_DebugService.ERROR_LEVEL_INFO + ": " + 
-                             consoleMessage.message + G_File.LINE_END_CHAR);
+                             consoleMessage.message + "\n");
       }
 
       return;
@@ -651,13 +640,10 @@ G_DebugService.prototype.observe = function(consoleMessage) {
  */
 G_DebugService.prototype.reportScriptError_ = function(message, sourceName, 
                                                        lineNumber, label) {
-  var message = ["",
-                 "------------------------------------------------------------",
-                 label + ": " + message,
-                 "location: " + sourceName + ", " + "line: " + lineNumber,
-                 "------------------------------------------------------------",
-                 "",
-                 ""].join(G_File.LINE_END_CHAR);
+  var message = "\n------------------------------------------------------------\n" +
+                label + ": " + message +
+                "\nlocation: " + sourceName + ", " + "line: " + lineNumber +
+                "\n------------------------------------------------------------\n\n";
 
   dump(message);
   this.maybeDumpToFile(message);
@@ -769,7 +755,7 @@ G_Loggifier.prototype.loggify = function(obj) {
           args[i] = arguments[i];
           argsString += (i == 0 ? "" : ", ");
           
-          if (isFunction(args[i])) {
+          if (typeof args[i] == "function") {
             argsString += "[function]";
           } else {
             argsString += args[i];
@@ -877,3 +863,15 @@ var G_debugService = new G_DebugService(); // Instantiate us!
 if (G_GDEBUG) {
   G_debugService.enableAllZones();
 }
+
+#else
+
+// Stubs for the debugging aids scattered through this component.
+// They will be expanded if you compile yourself a debug build.
+
+function G_Debug(who, msg) { }
+function G_Assert(who, condition, msg) { }
+function G_Error(who, msg) { }
+var G_debugService = { __noSuchMethod__: function() { } };
+
+#endif
